#include <unistd.h>

#include "StopWatch.h"
#include "SDLTools.h"
#include "SDLFrameRate.h"
#include "Configuration.h"
#include "Tiles.h"
#include "ObjectBase.h"
#include "MenuManager.h"
#include "SoundMixer.h"
#include "GameBridge.h"
#include "PlayerConfiguration.h"


void readCommandLine(int, char**);

Uint32 gfxflags = SDL_HWSURFACE | SDL_ANYFORMAT;
Uint32 sdlflags = SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE;
bool sound = true;

//----------------------------------------------------------------------------
int main(int argc, char** argv)
{
    try
    {
        Sint16 width = 640, height = 480;
        Uint8 depth = 16;
    
        const char* theme = "moagg2";
        const char* layout = "menu.xml";


        srand(time(NULL));
        readCommandLine(argc, argv);
        if (sound) sdlflags |= SDL_INIT_AUDIO;

        Configuration::getInstance()->searchDataDir();

        std::cout << "Initializing SDL ..." << std::flush;
        SDL_CALLS::Init(sdlflags);
        std::cout << "done." << std::endl;

        if (sound)
        {
            std::cout << "Initializing sound system ..." << std::flush;
            try
            {
                SoundMixer::init();
                std::cout << "done." << std::endl;
            }
            catch (Exception &e)
            {
                sound = false;
                std::cout << "failed." << std::endl;
                std::cout << "  (" << e << ")" << std::endl;
            }
        }

        std::cout << "Initializing tile structures ... " << std::flush;
        TileCategories::init();
        std::cout << "done." << std::endl;

        std::cout << "Initializing surfaces ... " << std::flush;
        SurfacesBase::init();
        std::cout << "done." << std::endl;
    
        std::cout << "Initializing configurations ... " << std::flush;
        PlayerConfiguration::create();
        std::cout << "done." << std::endl;

        std::cout << "Initializing GUI ... " << std::flush;
        MenuManager::init(width, height, depth, gfxflags, layout, theme);
        std::cout << "done." << std::endl;

        GameInterface::setInstance(GameBridge::getInstance());
    }
    catch (Exception &e)
    {
        std::cout << "Error initializing the game!" << std::endl
                  << e << "." << std::endl;
        return -1;
    }

    SDLFrameRate::setFrameRate(96);  // Always use 96 fps.

    PG_Application::GetApp()->Run();

    MenuManager::destroy();

    PlayerConfiguration::destroy();

    SurfacesBase::destroy();

    TileCategories::destroy();

    if (sound)
    {
        SoundMixer::destroy();
    }

    SDL_Quit();

    return 0;
}

//----------------------------------------------------------------------------
void printVersion()
{
    std::cout << "Moagg - Mother of all gravity games" <<
    std::endl << " (c) 2003-2004 Bernhard Trummer, Andreas Granig" <<
    std::endl;
}

//----------------------------------------------------------------------------
void printHelp(const char* me)
{
    printVersion();

    std::cout << 
    std::endl << "Usage: " << me << " [-h] [-v] [-s] [-f]" << 
    std::endl << "  -h: prints this message" <<
    std::endl << "  -v: prints version" <<
    std::endl << "  -s: disables sound" <<
    std::endl << "  -f: switches to fullscreen" <<
    std::endl;
}

//----------------------------------------------------------------------------
void readCommandLine(int argc, char** argv)
{
    int ch;
    char *argString = "hsfv";

    while ((ch = getopt(argc, argv, argString)) != EOF)
    {
        switch(ch)
        {
        case 'v':
            printVersion();
            exit(0);
            break;

        case 'h':
            printHelp(argv[0]);
            exit(0);
            break;

        case 's':
            sound = false;
            break;

        case 'f':
            gfxflags |= SDL_FULLSCREEN;
            break;
        }
    }
}
